

# FUNCTIONS tabulate counts/proportions and median(q1,q3) overall and by groups
## PRINT ONE LINE FOR VARIABLES WITH TWO CATEGORIES OR ONLY ONE OF INTEREST 
## EXAMPLE (PRINT FEMALE (%) -- NO NEED TO PRINT MALE(%) AS WELL)
# var must be a variable that takes on 1 value and the rest are NA
# similarly, rowlabel must be length 1
getnp1line <- function(var,by,rowlabel="",indent=TRUE,inc.pval=FALSE,by.order=FALSE,formatper="myper0",rowper=FALSE,noper=FALSE){
  rowlabel <- ifelse(rowlabel=="",label(var),rowlabel)
  if(rowlabel==""){stop("rowlabel was not specified, either as an attribute of var, or in the getnp1line call.")}
  myper1 <- get(formatper)
  n <- table(var,by)
  d <- table(by)
  n1 <- table(var)
  if(!rowper){p <- sapply(100*n[1,]/d,myper1)}
  if(rowper){p <- sapply(100*n[1,]/n1,myper1)}
  pt <- sapply(100*n1/length(by),myper1)
  if(indent==TRUE){line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(paste(n[1,]," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  else{line <- paste(paste(rowlabel),paste(paste(n[1,]," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  if(indent & noper){line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(n[1,],collapse=" & "),sum(n),sep=" & ")}
  if(!indent & noper){line <- paste(paste(rowlabel),paste(n[1,],collapse=" & "),sum(n),sep=" & ")}
  if(inc.pval==TRUE){
    var1 <- ifelse(is.na(var),0,var)
    if(by.order==TRUE){pval <- myformat4(wilcox.test(as.numeric(by)~ var1)$p.value)}
    if(by.order==FALSE){pval <- myformat4(chisq.test(table(var1,by))$p.value)}
    line <- paste(line,pval,sep=" & ")
  }
  return(list(line=line))
}

## PRINT MED (IQR) FOR VAR WITHIN EACH BY COLUMN AND OVERALL
# options include 'by.order'' which will affect the hypothesis test
# note will include \tnote{note} by the label
# OUTPUT INCLUDES line and line.incmiss (if there is missing data)
getmedtex <- function(var,by,label="",note=note,inc.pval=TRUE,lower.label=TRUE,miss.label=TRUE,by.order=FALSE,digits=8,formatper="myper0",iqrsep=" -- "){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getmedtex call.")}
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var),digits=digits)
  text <- paste(s[3]," (",s[2],iqrsep,s[5],")",sep="")
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s)); textby <- rep(NA,length(level))
  for(i in 1:length(level)){
    sby[i,1:5] <- round(summary(var[by==level[i]])[1:5],digits=digits)
    textby[i] <- ifelse(!is.na(sby[i,3]),paste(sby[i,3]," (",sby[i,2],iqrsep,sby[i,5],")",sep=""), "--")
  }
  if(inc.pval==TRUE){
    if(length(level)==1) {pval <- " "}
    else if(length(level)>1 & by.order==FALSE) {pval <- myformat4(kruskal.test(var ~ by)$p.value)}
    else if(length(level)>1 & by.order==TRUE)  {pval <- myformat4(cor.test(var, as.numeric(by),alternative="two.sided",method="spearman")$p.value)}
    
    if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,pval,sep=' & ')
    } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,pval,sep=' & ')
    }
  } else{
    if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,sep=' & ')
    } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,sep=' & ')
    }
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(lower.label==TRUE){line2 <- paste(paste("\\\\hspace{.1in}Missing",ifelse(miss.label,paste(" ",tolower(label),sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
    else{line2 <- paste(paste("\\\\hspace{.1in}Missing",ifelse(miss.label,paste(" ",label,sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}

## PRINT N (%) FOR VAR WITHIN EACH BY COLUMN AND OVERALL
# options include 'by.order' and 'var.order' which will affect the hypothesis test
# note will include \tnote{note} by the label
# OUTPUT INCLUDES line and line.incmiss (if there is missing data)
getnptex <- function(var,by,label="",rowlabel=rowlabel,note=note,inc.pval=TRUE,var.order=FALSE,by.order=FALSE,formatper="myper0",rowper=FALSE){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getnptex call.")}
  myper1 <- get(formatper)
  if(missing(rowlabel)){
    if(is.factor(var)){rowlabel <- levels(var)}
    if(is.character(var)){rowlabel <- names(table(var))}
  }
  if(length(rowlabel) != sum(!(unique(var)%in% NA))){warning("Function GETNPTEX: number of unique values does not match length of rowlabel")}
  n <- table(var,by)
  n1 <- rowSums(n)
  d <- colSums(table(var,by)[1:nrow(n),])
  p <- matrix(nrow=nrow(n), ncol=ncol(n))
  pt <- rep(NA,nrow(n))
  lines <- rep(NA,nrow(n))
  for(i in 1:nrow(n)){
    p[i,which(d==0)] <- 0
    if(!rowper){p[i,which(d!=0)] <- sapply(100*n[i,which(d!=0)]/d[which(d!=0)],myper1)}
    if(rowper){p[i,which(d!=0)] <- sapply(100*n[i,which(d!=0)]/n1[i],myper1)}
    pt[i] <- sapply(100*table(var)[i]/sum(table(var)[1:nrow(n)]),myper1)
    lines[i] <- paste(paste("\\\\hspace{.2in}",rowlabel[i]),paste(paste(n[i,]," (",p[i,],"\\\\%)",sep=""),collapse=" & "),paste(sum(n[i,])," (",pt[i],"\\\\%)",sep=""),sep=" & ")
  }
  # ADD PVALUE TO COLUMN HEADING IF DESIRED
  if(inc.pval==TRUE){
    if(by.order==TRUE){pval <- myformat4(kruskal.test(as.numeric(by), var)$p.value)}
    if(var.order==TRUE){pval <- myformat4(kruskal.test(var, by)$p.value)}
    if(by.order==TRUE & var.order==TRUE){pval <- myformat4(cor.test(var, by, alternative="two.sided", method="spearman")$p.value)} # ADDED 12/30/2010
    if(by.order==FALSE & var.order==FALSE){pval <- myformat4(chisq.test(n[n1!=0,d!=0])$p.value)}
    spaces <- rep(" ",1+length(table(by)))
    line1 <- paste(paste(label,", n(\\\\%)",sep=""),paste(spaces,collapse=" & "),pval,sep=" & ")
  } else{
    line1 <- paste(label,", n(\\\\%)",sep="")
  }
  all.lines <- paste(line1,paste(lines[which(n1!=0)],collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(missing(note)){ line2 <- paste("\\\\hspace{.1in}Missing",paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")
    } else{line2 <- paste(paste("\\\\hspace{.1in}Missing\\\\tnote{",note,"}",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")
    }
    all.lines.miss <- paste(line1,line2,paste(lines,collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
    return(list(line=all.lines,line.incmiss=all.lines.miss))
  }
  else{return(list(line=all.lines))}
}

getrangetex <- function(var,by,label="",by.order=FALSE,digits=8,formatper="myper0",rangesep=" -- "){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getrangetex call.")}
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var),digits=digits)
  text <- paste(s[1],rangesep,s[6],sep="")
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s)); textby <- rep(NA,length(level))
  for(i in 1:length(level)){
    sby[i,1:6] <- round(summary(var[by==level[i]])[1:6],digits=digits)
    textby[i] <- ifelse(!is.na(sby[i,3]),paste(sby[i,1],rangesep,sby[i,6],sep=""), "--")
  }
  line1 <- paste(paste0(label,", Range"),paste(textby,collapse=' & '),text,sep=' & ')
  return(list(line=line1))
}
## PRINT LATEX HEADER FOR GETNPTEX, GETMEDTEX, and GETNP1LINE functions
printheader <- function(crosstab,total=TRUE,total.label="Combined",pval=FALSE,pval.label="P-value"){
  if(total){
    x <- paste(c("",names(crosstab),total.label),collapse=" & ")
    y <- paste(c("",paste("(n=",c(crosstab,sum(crosstab)),")",sep="")),collapse=" & ")
  }
  if(!total){
    x <- paste(c("",names(crosstab)),collapse=" & ")
    y <- paste(c("",paste("(n=",c(crosstab),")",sep="")),collapse=" & ")
  }
  if(pval){x <- paste(c(x,pval.label),collapse=" & ")}
  line <- paste(c(x,y),collapse=" \\\\\\\\ ")
  return(line)
}
myformat2 <- function(x){format(round(x,2),nsmall=2)}
myformat4 <- function(x){
  y <- NULL
  if(x < 0.001){y <- "<0.001"}
  else{y <- format(round(x,3),nsmall=3)}
  return(y)
}
myper0 <- function(x){
  y <- NULL
  if(x <= 0.5 & x!=0){y <- "<1"}
  else{y <- format(round(x),nsmall=0)}
  return(y)
}

convertdate1 <- function(mydate){as.Date(mydate,"%Y-%m-%d")}
convertdate <- function(mydate){as.Date(mydate,"%m/%d/%Y")}
